#ifndef COMMON_H_
#define COMMON_H_

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/kthread.h>
#include <linux/string.h>
#include <linux/time.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/errno.h>
#include <linux/slab.h>


#include <opentk/OpenTk_Common.h>


#define MIN(a, b)    ( ( (a) < (b) ) ? (a) : (b) )
#define MAX(a, b)    ( ( (a) > (b) ) ? (a) : (b) )


#define SAFE_VFREE(p) \
   do{ if(p) {vfree(p); (p)=NULL;} } while(0)

#define SAFE_KFREE(p) \
   do{ if(p) {kfree(p); (p)=NULL;} } while(0)

#define SAFE_DESTRUCT(p, destructor) \
   do{if(p) {destructor(p); (p)=NULL;} } while(0)


// module name (used for logging)
#define FHGFS_MODULE_NAME_STR "fhgfs_opentk"

// a printk-version that adds the module name and comm name
#define printk_fhgfs(levelStr, fmtStr, ...) \
   printk(levelStr FHGFS_MODULE_NAME_STR ": " "%s: " fmtStr, current->comm, ## __VA_ARGS__)

// for interrupt handling routines (does not print "current")
#define printk_fhgfs_ir(levelStr, fmtStr, ...) \
   printk(levelStr FHGFS_MODULE_NAME_STR ": " fmtStr, ## __VA_ARGS__)


#ifdef LOG_DEBUG_MESSAGES

#define printk_fhgfs_debug(levelStr, fmtStr, ...) \
   printk_fhgfs(levelStr, fmtStr, ## __VA_ARGS__)
#define printk_fhgfs_ir_debug(levelStr, fmtStr, ...) \
   printk_fhgfs_ir(levelStr, fmtStr, ## __VA_ARGS__)

#else

#define printk_fhgfs_debug(levelStr, fmtStr, ...) /* logging of debug messages disabled */
#define printk_fhgfs_ir_debug(levelStr, fmtStr, ...) /* logging of debug messages disabled */

#endif // LOG_DEBUG_MESSAGES


#ifdef FHGFS_OPENTK_LOG_CONN_ERRORS
   #define printk_fhgfs_connerr(levelStr, fmtStr, ...) \
      printk_fhgfs(levelStr, fmtStr, ## __VA_ARGS__)
#else
   #define printk_fhgfs_connerr(levelStr, fmtStr, ...) /* logging of conn errors disabled */
#endif // FHGFS_OPENTK_LOG_CONN_ERRORS


// this macro mutes warnings about unused variables
#define IGNORE_UNUSED_VARIABLE(a)   do{ if( ((long)a)==1) {} } while(0)

// mute false alarm for __copy_to_user return value check
#define IGNORE_RETURN_VALUE(a)      IGNORE_UNUSED_VARIABLE(a)

/*
 * Debug definitions:
 * - LOG_DEBUG_MESSAGES: Enables logging of some extra debug messages that will not be
 *    available otherwise.
 */


#endif /*COMMON_H_*/
