/*
 * Wrapper functions for EXPORT_SYMBOL_GPL kernel functions
 */

#include <opentk/OpenTk_Common.h>
#include <common/Common.h>
#include <linux/mount.h>
#include <linux/fs.h>
#include <linux/pagemap.h>

/**
 * Wrapper function for mnt_want_write()
 */
int os_mnt_want_write(struct vfsmount *mnt)
{
   #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
      if (mnt->mnt_sb->s_flags & MS_RDONLY)
         return -EROFS;
      else
         return 0;
   #else
      // >= 2.6.26
      return mnt_want_write(mnt);
   #endif

}
EXPORT_SYMBOL(os_mnt_want_write);

/**
 * Wrapper function for mnt_drop_write()
 */
void os_mnt_drop_write(struct vfsmount *mnt)
{
   #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
      // noop before 2.6.26, as those didn't have mnt_want_write yet and so didn't take a
      // write ref-counter yet.
      return;
   #else
      // >= 2.6.26
      return mnt_drop_write(mnt);
   #endif

}
EXPORT_SYMBOL(os_mnt_drop_write);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,30)
/**
 * Wrapper for add_to_page_cache_lru()
 */
int os_addToPageCacheLRU(struct page *page, struct address_space *mapping,
   pgoff_t offset, gfp_t gfp_mask)
{
   return add_to_page_cache_lru(page, mapping, offset, gfp_mask);
}
EXPORT_SYMBOL(os_addToPageCacheLRU);
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
int os_GenericDropInode(struct inode *inode)
{
   return generic_drop_inode(inode);
}
#else
void os_GenericDropInode(struct inode *inode)
{
   generic_drop_inode(inode);
}
#endif
EXPORT_SYMBOL(os_GenericDropInode);

